searchData={"items":[{"type":"module","title":"Electric.Phoenix","doc":"Wrappers to ease integration of [Electric’s Postgres syncing\nservice](https://electric-sql.com) with [Phoenix\napplications](https://www.phoenixframework.org/).\n\nThere are currently 2 integration modes: [`Phoenix.LiveView`\nstreams](#module-phoenix-liveview-streams) and [configuration\ngateway](#module-configuration-gateway).","ref":"Electric.Phoenix.html"},{"type":"module","title":"Phoenix.LiveView Streams - Electric.Phoenix","doc":"`Electric.Phoenix.LiveView.electric_stream/4` integrates with\n[`Phoenix.LiveView.stream/4`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#stream/4)\nand provides a live updating collection of items.","ref":"Electric.Phoenix.html#module-phoenix-liveview-streams"},{"type":"module","title":"Configuration Gateway - Electric.Phoenix","doc":"Using `Electric.Phoenix.Plug` you can create endpoints that\nreturn configuration information for your Electric Typescript clients. See\n[that module's documentation](`Electric.Phoenix.Plug`) for\nmore information.","ref":"Electric.Phoenix.html#module-configuration-gateway"},{"type":"module","title":"Installation - Electric.Phoenix","doc":"Add `electric_phoenix` to your application dependencies:\n\n    def deps do\n      [\n        {:electric_phoenix, \"~> 0.1\"}\n      ]\n    end","ref":"Electric.Phoenix.html#module-installation"},{"type":"module","title":"Configuration - Electric.Phoenix","doc":"In your `config/config.exs` or `config/runtime.exs` you **must** configure the\nendpoint for the Electric streaming API:\n\n    import Config\n\n    config :electric_phoenix,\n      # required\n      electric_url: System.get_env(\"ELECTRIC_URL\", \"http://localhost:3000\"),\n      # optional\n      database_id: System.get_env(\"ELECTRIC_DATABASE_ID\", nil)","ref":"Electric.Phoenix.html#module-configuration"},{"type":"function","title":"Electric.Phoenix.client!/0","doc":"Create a new `Electric.Client` instance based on the application config.","ref":"Electric.Phoenix.html#client!/0"},{"type":"type","title":"Electric.Phoenix.shape_definition/0","doc":"","ref":"Electric.Phoenix.html#t:shape_definition/0"},{"type":"module","title":"Electric.Phoenix.Component","doc":"","ref":"Electric.Phoenix.Component.html"},{"type":"module","title":"Electric.Phoenix.LiveView","doc":"","ref":"Electric.Phoenix.LiveView.html"},{"type":"function","title":"Electric.Phoenix.LiveView.electric_client_configuration/1","doc":"Embed client configuration for a shape into your HTML.\n\n     \n\nThis will put a ` ` tag into your page setting\n`window.todo_shape_config` to the configuration needed to subscribe to\nchanges to the `MyApp.Todo` table.\n\n     \n      window.todo_shape_config = {\"url\":\"https://localhost:3000/v1/shape/todos\" /* , ... */}\n     \n\nIf you include a `:script` slot then you have complete control over how the\nconfiguration is applied.\n\n     \n      <:script :let={configuration}>\n        const container = document.getElementById(\"root\")\n        const root = createRoot(container)\n        root.render(\n          React.createElement(\n            MyApp, {\n              client_config: <%= configuration %>\n            },\n            null\n          )\n        );\n      </:script>\n     \n\nThe `configuration` variable in the `:script` block is the  JSON-encoded\nclient configuration.\n\n     \n      const container = document.getElementById(\"root\")\n      const root = createRoot(container)\n      root.render(\n        React.createElement(\n          MyApp, {\n            client_config: {\"url\":\"https://localhost:3000/v1/shape/todos\" /* , ... */}\n          },\n          null\n        )\n      );","ref":"Electric.Phoenix.LiveView.html#electric_client_configuration/1"},{"type":"function","title":"Attributes - Electric.Phoenix.LiveView.electric_client_configuration/1","doc":"* `shape` (`:any`) (required) - The Ecto query (or schema module) to subscribe to.\n* `key` (`:string`) - The key in the top-level `window` object to put the configuration object. Defaults to `\"electric_client_config\"`.\n* `client` (`:any`) - Optional client. If not set defaults to `Electric.Phoenix.client!()`.","ref":"Electric.Phoenix.LiveView.html#electric_client_configuration/1-attributes"},{"type":"function","title":"Slots - Electric.Phoenix.LiveView.electric_client_configuration/1","doc":"* `script` - An optional inner block that allows you to override what you want to do with the configuration JSON.","ref":"Electric.Phoenix.LiveView.html#electric_client_configuration/1-slots"},{"type":"function","title":"Electric.Phoenix.LiveView.electric_stream/4","doc":"Maintains a LiveView stream from the given Ecto query.\n\n- `name` The name to use for the LiveView stream.\n- `query` An [`Ecto`](`Ecto`) query that represents the data to stream from the database.\n\nFor example:\n\n    def mount(_params, _session, socket) do\n      socket =\n        Electric.Phoenix.LiveView.electric_stream(\n          socket,\n          :admins,\n          from(u in Users, where: u.admin == true)\n        )\n      {:ok, socket}\n    end\n\nThis will subscribe to the configured Electric server and keep the list of\n`:admins` in sync with the database via a `Phoenix.LiveView` stream.\n\nUpdates will be delivered to the view via messages to the LiveView process.\n\nTo handle these you need to add a `handle_info/2` implementation that receives these:\n\n    def handle_info({:electric, event}, socket) do\n      {:noreply, Electric.Phoenix.LiveView.electric_stream_update(socket, event)}\n    end\n\nSee the docs for\n[`Phoenix.LiveView.stream/4`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#stream/4)\nfor details on using LiveView streams.","ref":"Electric.Phoenix.LiveView.html#electric_stream/4"},{"type":"function","title":"Lifecycle Events - Electric.Phoenix.LiveView.electric_stream/4","doc":"Most `{:electric, event}` messages are opaque and should be passed directly\nto the `electric_stream_update/3` function, but there are two events that are\noutside Electric's replication protocol and designed to be useful in the\nLiveView component.\n\n- `{:electric, {stream_name, :loaded}}` - sent when the Electric event stream has passed\n  from initial state to update mode.\n\n  This event is useful to show the stream component after\n  the initial sync. Because of the streaming nature of Electric Shapes, the\n  intitial sync can cause flickering as items are added, removed and updated.\n\n  E.g.:\n\n      # in the LiveView component\n      def handle_info({:electric, {_name, :live}}, socket) do\n        {:noreply, assign(socket, :show_stream, true)}\n      end\n\n      # in the template\n       \n         \n          <%= item.value %>\n         \n       \n\n- `{:electric, {stream_name, :live}}` - sent when the Electric stream is in\n  `live` mode, that is the initial state has loaded and the client is\n  up-to-date with the database and is long-polling for new events from the\n  Electric server.\n\nIf your app doesn't need this extra information, then you can ignore them and\njust have a catch-all callback:\n\n    def handle_info({:electric, event}, socket) do\n      {:noreply, Electric.Phoenix.LiveView.electric_stream_update(socket, event)}\n    end\n\n`Electric.Phoenix.LiveView.electric_stream_update` will just ignore the\nlifecycle events.","ref":"Electric.Phoenix.LiveView.html#electric_stream/4-lifecycle-events"},{"type":"function","title":"Sub-components - Electric.Phoenix.LiveView.electric_stream/4","doc":"If you register your Electric stream in a sub-component you will still\nreceive Electric messages in the LiveView's root/parent process.\n\n`Electric.Phoenix` handles this for you by encapsulating component messages\nso it can correctly forward on the event to the component.\n\nSo in the parent `LiveView` process you handle the `:electric` messages as\nabove:\n\n    defmodule MyLiveView do\n      use Phoenix.LiveView\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <.live_component id=\"my_component\" module={MyComponent} />\n         \n        \"\"\"\n      end\n\n      # We setup the Electric electric_stream in the component but update messages will\n      # be sent to the parent process.\n      def handle_info({:electric, event}, socket) do\n        {:noreply, Electric.Phoenix.LiveView.electric_stream_update(socket, event)}\n      end\n    end\n\nIn the component you must handle these events in the\n`c:Phoenix.LiveComponent.update/2` callback:\n\n    defmodule MyComponent do\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n           \n            <%= user.name %>\n           \n         \n        \"\"\"\n      end\n\n      # Equivalent to the `handle_info({:electric, {stream_name, :live}}, socket)` callback\n      # in the parent LiveView.\n      def update(%{electric: {_stream_name, :live}}, socket) do\n        {:ok, socket}\n      end\n\n      # Equivalent to the `handle_info({:electric, event}, socket)` callback\n      # in the parent LiveView.\n      def update(%{electric: event}, socket) do\n        {:ok, Electric.Phoenix.LiveView.electric_stream_update(socket, event)}\n      end\n\n      def update(assigns, socket) do\n        {:ok, Electric.Phoenix.electric_stream(socket, :users, User)}\n      end\n    end","ref":"Electric.Phoenix.LiveView.html#electric_stream/4-sub-components"},{"type":"function","title":"Electric.Phoenix.LiveView.electric_stream_update/3","doc":"Handle Electric events within a LiveView.\n\n    def handle_info({:electric, event}, socket) do\n      {:noreply, Electric.Phoenix.LiveView.electric_stream_update(socket, event, at: 0)}\n    end\n\nThe `opts` are passed to the `Phoenix.LiveView.stream_insert/4` call.","ref":"Electric.Phoenix.LiveView.html#electric_stream_update/3"},{"type":"opaque","title":"Electric.Phoenix.LiveView.component_event/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:component_event/0"},{"type":"type","title":"Electric.Phoenix.LiveView.event/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:event/0"},{"type":"opaque","title":"Electric.Phoenix.LiveView.replication_event/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:replication_event/0"},{"type":"opaque","title":"Electric.Phoenix.LiveView.root_event/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:root_event/0"},{"type":"type","title":"Electric.Phoenix.LiveView.state_event/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:state_event/0"},{"type":"type","title":"Electric.Phoenix.LiveView.stream_option/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:stream_option/0"},{"type":"type","title":"Electric.Phoenix.LiveView.stream_options/0","doc":"","ref":"Electric.Phoenix.LiveView.html#t:stream_options/0"},{"type":"module","title":"Electric.Phoenix.Plug","doc":"Provides an configuration endpoint for use in your Phoenix applications.\n\nRather than configuring your [Electric Typescript\nclient](https://electric-sql.com/docs/api/clients/typescript) directly, you\ninstead configure a route in your application with a pre-configured\n`Electric.Client.ShapeDefinition` and then retreive the URL and other\nconfiguration for that shape from your client via a request to your Phoenix\napplication.\n\nIn your Phoenix application, [add a route](https://hexdocs.pm/phoenix/Phoenix.Router.html) to\n`Electric.Phoenix.Plug` specifying a particular shape:\n\n    defmodule MyAppWeb.Router do\n      scope \"/shapes\" do\n        pipe_through :browser\n\n        get \"/todos\", Electric.Phoenix.Plug,\n          shape: Electric.Client.shape!(\"todos\", where: \"visible = true\")\n      end\n    end\n\nThen in your client code, you retrieve the shape configuration directly\nfrom the Phoenix endpoint:\n\n``` typescript\nimport { ShapeStream } from '@electric-sql/client'\n\nconst endpoint = `https://localhost:4000/shapes/todos`\nconst response = await fetch(endpoint)\nconst config = await response.json()\n\n// The returned `config` has all the information you need to subscribe to\n// your shape\nconst stream = new ShapeStream(config)\n\nstream.subscribe(messages => {\n  // ...\n})\n```\n\nYou can add additional authentication/authorization for shapes using\n[Phoenix's\npipelines](https://hexdocs.pm/phoenix/Phoenix.Router.html#pipeline/2) or\nother [`plug`\ncalls](https://hexdocs.pm/phoenix/Phoenix.Router.html#plug/2).","ref":"Electric.Phoenix.Plug.html"},{"type":"module","title":"Plug.Router - Electric.Phoenix.Plug","doc":"For  pure `Plug`-based applications, you can use `Plug.Router.forward/2`:\n\n    defmodule MyRouter do\n      use Plug.Router\n\n      plug :match\n      plug :dispatch\n\n      forward \"/shapes/items\",\n        to: Electric.Phoenix.Plug,\n        shape: Electric.Client.shape!(\"items\")\n\n      match _ do\n        send_resp(conn, 404, \"oops\")\n      end\n    end","ref":"Electric.Phoenix.Plug.html#module-plug-router"},{"type":"module","title":"Parameter-based shapes - Electric.Phoenix.Plug","doc":"As well as defining fixed-shapes for a particular url, you can request\nshape configuration using parameters in your request:\n\n    defmodule MyAppWeb.Router do\n      scope \"/\" do\n        pipe_through :browser\n\n        get \"/shape\", Electric.Phoenix.Plug, []\n      end\n    end\n\n``` typescript\nimport { ShapeStream } from '@electric-sql/client'\n\nconst endpoint = `https://localhost:4000/shape?table=items&namespace=public&where=visible%20%3D%20true`\nconst response = await fetch(endpoint)\nconst config = await response.json()\n\n// as before\n```\n\nThe parameters are:\n\n- `table` - The Postgres table name (required).\n- `namespace` - The Postgres schema if not specified defaults to `public`.\n- `where` - The [where clause](https://electric-sql.com/docs/guides/shapes#where-clause) to filter items from the shape.","ref":"Electric.Phoenix.Plug.html#module-parameter-based-shapes"},{"type":"function","title":"Electric.Phoenix.Plug.call/2","doc":"","ref":"Electric.Phoenix.Plug.html#call/2"},{"type":"function","title":"Electric.Phoenix.Plug.send_configuration/3","doc":"Send the client configuration for a given shape to the browser.","ref":"Electric.Phoenix.Plug.html#send_configuration/3"},{"type":"function","title":"Example - Electric.Phoenix.Plug.send_configuration/3","doc":"get \"/my-shapes/messages\" do\n      user_id = get_session(conn, :user_id)\n      shape = from(m in Message, where: m.user_id == ^user_id)\n      Electric.Phoenix.Plug.send_configuration(conn, shape)\n    end\n\n    get \"/my-shapes/tasks/:project_id\" do\n      project_id = conn.params[\"project_id\"]\n\n      if user_has_access_to_project?(project_id) do\n        shape = where(Task, project_id: ^project_id)\n        Electric.Phoenix.Plug.send_configuration(conn, shape)\n      else\n        send_resp(conn, :forbidden, \"You do not have permission to view project #{project_id}\")\n      end\n    end","ref":"Electric.Phoenix.Plug.html#send_configuration/3-example"},{"type":"function","title":"Electric.Phoenix.Plug.shape!/2","doc":"Defines a shape based on a root `Ecto` query plus some filters based on the\ncurrent request.\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      shape: Electric.Phoenix.Plug.shape!(\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      )\n\nThe `params` describe the way to build the `where` clause on the shape from\nthe request parameters.\n\nFor example, `[id: :user_id]` means that the `id` column on the table should\nmatch the value of the `user_id` parameter, equivalent to:\n\n    from(\n      t in Table,\n      where: t.id == ^conn.params[\"user_id\"]\n    )\n\nIf both the table column and the request parameter have the same name, then\nyou can just pass the name, so:\n\n    Electric.Phoenix.Plug.shape!(Table, [:visible])\n\nis equivalent to:\n\n    from(\n      t in Table,\n      where: t.visible == ^conn.params[\"visible\"]\n    )\n\nIf you need to match on something other than `==` then you can pass the operator in the params:\n\n    Electric.Phoenix.Plug.shape!(Table, size: [>=: :size])\n\nis equivalent to:\n\n    from(\n      t in Table,\n      where: t.size >= ^conn.params[\"size\"]\n    )\n\nInstead of calling `shape!/2` directly in your route definition, you can just\npass a list of `[query | params]` to do the same thing:\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      shape: [\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      ]","ref":"Electric.Phoenix.Plug.html#shape!/2"},{"type":"type","title":"Electric.Phoenix.Plug.conn_param_spec/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:conn_param_spec/0"},{"type":"type","title":"Electric.Phoenix.Plug.dynamic_shape_param/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:dynamic_shape_param/0"},{"type":"type","title":"Electric.Phoenix.Plug.dynamic_shape_params/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:dynamic_shape_params/0"},{"type":"type","title":"Electric.Phoenix.Plug.op/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:op/0"},{"type":"type","title":"Electric.Phoenix.Plug.param_name/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:param_name/0"},{"type":"type","title":"Electric.Phoenix.Plug.table_column/0","doc":"","ref":"Electric.Phoenix.Plug.html#t:table_column/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}